@@ -547,6 +547,11 @@ int lftp_ssl_gnutls::do_handshake()
 
    return DONE;
 }
+
+#ifndef GNUTLS_E_PREMATURE_TERMINATION // for gnutls < 3.0
+# define GNUTLS_E_PREMATURE_TERMINATION GNUTLS_E_UNEXPECTED_PACKET_LENGTH
+#endif
+
 int lftp_ssl_gnutls::read(char *buf,int size)
 {
    if(error)
@@ -560,9 +565,9 @@ int lftp_ssl_gnutls::read(char *buf,int size)
    {
       if(res==GNUTLS_E_AGAIN || res==GNUTLS_E_INTERRUPTED)
 	 return RETRY;
-      else if(res==GNUTLS_E_UNEXPECTED_PACKET_LENGTH)
+      else if(res==GNUTLS_E_UNEXPECTED_PACKET_LENGTH || res==GNUTLS_E_PREMATURE_TERMINATION)
       {
-	 Log::global->Format(7,"gnutls_record_recv: %s; assuming EOF\n",gnutls_strerror(res));
+	 Log::global->Format(7,"gnutls_record_recv: %s Assuming EOF.\n",gnutls_strerror(res));
 	 return 0;
       }
       else // error
@@ -610,7 +615,8 @@ void lftp_ssl_gnutls::copy_sid(const lftp_ssl_gnutls *o)
 {
    size_t session_data_size;
    void *session_data;
-   if(gnutls_session_get_data(o->session,NULL,&session_data_size)!=GNUTLS_E_SUCCESS)
+   int res=gnutls_session_get_data(o->session,NULL,&session_data_size);
+   if(res!=GNUTLS_E_SUCCESS && res!=GNUTLS_E_SHORT_MEMORY_BUFFER)
       return;
    session_data=xmalloc(session_data_size);
    if(gnutls_session_get_data(o->session,session_data,&session_data_size)!=GNUTLS_E_SUCCESS)
@@ -762,8 +768,6 @@ lftp_ssl_openssl_instance::lftp_ssl_openssl_instance()
 #endif
 
    RAND_file_name(file,sizeof(file));
-   if(RAND_egd(file)>0)
-      return;
 
    if(RAND_load_file(file,-1) && RAND_status()!=0)
       atexit(lftp_ssl_write_rnd);
